@@ -1,9 +1,9 @@
 #
 # -*- Perl -*-
-# $Id: olemsword.pl,v 1.8.4.10 2005/06/06 07:07:34 opengl2772 Exp $
+# $Id: olemsword.pl,v 1.8.4.19 2007-11-18 10:19:52 opengl2772 Exp $
 # Copyright (C) 1999 Jun Kurabe,
 #		1999-2000 Ken-ichi Hirose,
-#               2004-2005 Namazu Project All rights reserved.
+#               2004-2007 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -52,19 +52,57 @@ no strict 'refs';  # for symbolic reference: $fields;
 require 'util.pl';
 require 'gfilter.pl';
 
+my $version = 0;
+
 sub mediatype() {
-    return ('application/msword');
+    status();
+
+    if ($version >= 9) {
+        # 12.0 Office 2007
+        # 11.0 Office 2003
+        # 10.0 Office 2002
+        #  9.0 Office 2000
+        return (
+            'application/msword',
+            'application/vnd.openxmlformats-officedocument.wordprocessingml',
+        );
+    } else {
+        #  8.0 Office 97, 98
+        return (
+            'application/msword',
+        );
+    }
 }
 
 sub status() {
+    my $const = undef;
+
     open (SAVEERR,">&STDERR");
     open (STDERR,">nul");
-    my $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 11.0 Object Library");
-    $const = Win32::OLE::Const->Load("Microsoft Word 10.0 Object Library") unless $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 9.0 Object Library") unless $const;
-    $const = Win32::OLE::Const->Load("Microsoft Word 8.0 Object Library") unless $const;
+
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 12.0 Object Library");
+        $version = 12;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 11.0 Object Library");
+        $version = 11;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 10.0 Object Library");
+        $version = 10;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 9.0 Object Library");
+        $version = 9;
+    }
+    if (!defined $const) {
+        $const = Win32::OLE::Const->Load("Microsoft Word 8.0 Object Library");
+        $version = 8;
+    }
+
     open (STDERR,">&SAVEERR");
+
     return 'yes' if (defined $const);
     return 'no';
 }
@@ -84,7 +122,28 @@ sub post_codeconv () {
 sub add_magic ($) {
     my ($magic) = @_;
 
+    # FIXME: very ad hoc.
     $magic->addFileExts('\\.doc$', 'application/msword');
+    $magic->addFileExts('\\.docx$', 'application/vnd.openxmlformats-officedocument.wordprocessingml');
+
+    # FIXME: very ad hoc. Excel
+    $magic->addFileExts('\\.xls$', 'application/excel');
+    $magic->addFileExts('\\.xlsx$', 'application/vnd.openxmlformats-officedocument.spreadsheetml');
+    $magic->addFileExts('\\.xlsm$', 'application/vnd.ms-excel.sheet.macroEnabled');
+    $magic->addFileExts('\\.xlsb$', 'application/vnd.ms-excel.sheet.binary.macroEnabled');
+    # FIXME: very ad hoc. PowerPoint
+    $magic->addFileExts('\\.pp[st]$', 'application/powerpoint');
+    $magic->addFileExts('\\.pp[st]x$', 'application/vnd.openxmlformats-officedocument.presentationml');
+    # FIXME: very ad hoc. Visio
+    $magic->addFileExts('\\.vs[dst]$', 'application/vnd.visio');
+    $magic->addFileExts('\\.v[dst]x$', 'application/vnd.visio');
+    # FIXME: very ad hoc. Ichitaro
+    $magic->addFileExts('\\.jsw$', 'application/ichitaro4');
+    $magic->addFileExts('\\.jaw$|\\.jtw$', 'application/ichitaro5');
+    $magic->addFileExts('\\.jbw$|\\.juw$', 'application/ichitaro6');
+    $magic->addFileExts('\\.jfw$|\\.jvw$', 'application/ichitaro7');
+    $magic->addFileExts('\\.jtd$|\\.jtt$', 'application/x-js-taro');
+
     return;
 }
 
@@ -238,7 +297,8 @@ sub ReadMSWord ($$$$) {
     # Redirect stderr to null device, to ignore Error and Exception message.
     open (SAVEERR,">&STDERR");
     open (STDERR,">nul");
-    # Load Office 97/98/2000/XP/2003 Constant
+    # Load Office 97/98/2000/XP/2003/2007 Constant
+    $office_consts = Win32::OLE::Const->Load("Microsoft Office 12.0 Object Library") unless $office_consts;
     $office_consts = Win32::OLE::Const->Load("Microsoft Office 11.0 Object Library") unless $office_consts;
     $office_consts = Win32::OLE::Const->Load("Microsoft Office 10.0 Object Library") unless $office_consts;
     $office_consts = Win32::OLE::Const->Load("Microsoft Office 9.0 Object Library") unless $office_consts;
@@ -317,6 +377,7 @@ sub getFrames ($$) {
 
     my $enum_func = sub {
 	my $obj = shift;
+        return 1 if (!defined($obj->Range));
 	my $p = $obj->Range->{Text};
 #	chop $p;
         $$cont .= "$p\n" if ( defined $p );
