@@ -1,8 +1,8 @@
 #
 # -*- Perl -*-
-# $Id: olevisio.pl,v 1.4.2.3 2005/06/06 07:07:34 opengl2772 Exp $
-# Copyright (C) 2004-2005 Tadamasa Teranishi,
-#               2004-2005 Namazu Project All rights reserved.
+# $Id: olevisio.pl,v 1.4.2.6 2007-01-14 04:12:04 opengl2772 Exp $
+# Copyright (C) 2004-2007 Tadamasa Teranishi,
+#               2004-2007 Namazu Project All rights reserved.
 #     This is free software with ABSOLUTELY NO WARRANTY.
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -67,11 +67,15 @@ sub status() {
 
     open (SAVEERR, ">&STDERR");
     open (STDERR, ">nul");
-    $const = Win32::OLE::Const->Load("Microsoft Visio 11.0 Type Library");
+    $const = Win32::OLE::Const->Load("Microsoft Visio 12.0 Type Library");
+    $const = Win32::OLE::Const->Load("Microsoft Visio 11.0 Type Library")
+        unless $const;
     $const = Win32::OLE::Const->Load("Microsoft Visio 2002 Type Library")
         unless $const;
     $const = Win32::OLE::Const->Load("Microsoft Visio 2000 Type Library")
         unless $const;
+    $const = Win32::OLE::Const->Load("Visio 2000 Type Library")
+        unless $const;
     open (STDERR, ">&SAVEERR");
     return 'yes' if (defined $const);
     return 'no';
@@ -91,6 +95,8 @@ sub post_codeconv () {
 
 sub add_magic ($) {
     my ($magic) = @_;
+
+    # FIXME: very ad hoc.
     $magic->addFileExts('\\.vs[dst]$', 'application/vnd.visio');
     $magic->addFileExts('\\.v[dst]x$', 'application/vnd.visio');
     return;
