@@ -2,10 +2,10 @@
  * 
  * hlist.c -
  * 
- * $Id: hlist.c,v 1.51.4.17 2005/10/26 00:47:08 opengl2772 Exp $
+ * $Id: hlist.c,v 1.51.4.22 2008-03-06 15:38:08 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2005 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,10 +26,7 @@
  * This file must be encoded in EUC-JP encoding
  * 
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <errno.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,6 +34,16 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#include <math.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -46,9 +53,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -60,6 +67,7 @@
 #include "search.h"
 #include "query.h"
 #include "score.h"
+
 static int document_number = 0;  /* Number of documents covered in a target index */
 static char field_for_sort[BUFSIZE] = "";  /* field_for_sort name used with sorting */
 
@@ -102,7 +110,8 @@ field_sort(NmzResult hlist)
 
     for (i = 0; i < hlist.num; i++) {
 	char buf[BUFSIZE];
-	int leng;
+	size_t leng;
+
 	nmz_get_field_data(hlist.data[i].idxid, 
 			   hlist.data[i].docid, field_for_sort, buf);
 	nmz_chomp(buf);
@@ -279,7 +288,10 @@ nmz_andmerge(NmzResult left, NmzResult right, int *ignore)
 		break;
 	    if (left.data[i].docid == right.data[j].docid) {
 
-		nmz_copy_hlist(left, v, left, i);
+                if (v != i) {
+                    nmz_copy_hlist(left, v, left, i);
+                }
+
                 if (nmz_is_tfidfmode()) {
                     left.data[v].score = 
 			left.data[i].score + right.data[j].score;
@@ -341,7 +353,9 @@ nmz_notmerge(NmzResult left, NmzResult right, int *ignore)
 	    }
 	}
 	if (!f) {
-	    nmz_copy_hlist(left, v, left, i);
+            if (v != i) {
+	        nmz_copy_hlist(left, v, left, i);
+            }
 	    v++;
 	}
     }
@@ -384,8 +398,11 @@ nmz_ormerge(NmzResult left, NmzResult right)
     n = left.num + right.num;
 
     nmz_malloc_hlist(&val, n);
-    if (val.stat == ERR_FATAL)
+    if (val.stat == ERR_FATAL) {
+	nmz_free_hlist(left);
+	nmz_free_hlist(right);
         return val;
+    }
 
     for (v = 0, i = 0, j = 0; i < left.num; i++) {
 	for (; j < right.num; j++) {
@@ -427,6 +444,10 @@ nmz_ormerge(NmzResult left, NmzResult right)
 void 
 nmz_malloc_hlist(NmzResult * hlist, int n)
 {
+    hlist->num  = 0;
+    hlist->data = NULL;
+    hlist->stat = SUCCESS;
+
     if (n <= 0) return;
     hlist->data = malloc(n * sizeof(struct nmz_data));
     if (hlist->data == NULL) {
@@ -455,8 +476,6 @@ nmz_free_hlist(NmzResult hlist)
 {
     if (hlist.stat != SUCCESS || hlist.num <= 0) return;
     free(hlist.data);
-    hlist.data = NULL;
-    hlist.num = 0;
 }
 
 void 
@@ -494,8 +513,15 @@ nmz_merge_hlist(NmzResult *hlists)
         }
     }
     nmz_malloc_hlist(&value, n);
-    if (value.stat == ERR_FATAL)
+    if (value.stat == ERR_FATAL) {
+        for(i = 0; i < nmz_get_idxnum(); i++) {
+            if (hlists[i].stat != SUCCESS || hlists[i].num <= 0)
+                continue;
+            nmz_free_hlist(hlists[i]);
+        }
         return value;
+    }
+
     for(i = n = 0; i < nmz_get_idxnum(); i++) {
         if (hlists[i].stat != SUCCESS || hlists[i].num <= 0) 
             continue;
@@ -515,7 +541,7 @@ NmzResult
 nmz_do_date_processing(NmzResult hlist)
 {
     FILE *date_index;
-    int i;
+    int i, v;
 
     date_index = fopen(NMZ.t, "rb");
     if (date_index == NULL) {
@@ -524,9 +550,9 @@ nmz_do_date_processing(NmzResult hlist)
         return hlist; /* error */
     }
 
-    for (i = 0; i < hlist.num ; i++) {
-        if (fseek(date_index, 
-	        hlist.data[i].docid * sizeof(hlist.data[i].date), 0) != 0) 
+    for (i = 0; i < hlist.num; i++) {
+        if (fseek(date_index,
+                hlist.data[i].docid * sizeof(hlist.data[i].date), 0) != 0)
 	{
 	    nmz_set_dyingmsg(nmz_msg("%s: %s", NMZ.t, strerror(errno)));
 	    hlist.stat = ERR_FATAL;
@@ -535,21 +561,22 @@ nmz_do_date_processing(NmzResult hlist)
         }
         nmz_fread(&hlist.data[i].date, 
 		  sizeof(hlist.data[i].date), 1, date_index);
-
-        if (hlist.data[i].date == -1) {  
-            /* The missing number, this document has been deleted */
-            int j;
-
-            for (j = i + 1; j < hlist.num; j++) { /* shift */
-                nmz_copy_hlist(hlist, j - 1, hlist, j);
-            }
-            hlist.num--;
-            i--;
-        }
     }
 
     fclose(date_index);
 
+    for (v = 0, i = 0; i < hlist.num; i++) {
+        if (hlist.data[i].date == -1) {  
+            /* The missing number, this document has been deleted */
+        } else {
+            if (v != i) {
+                nmz_copy_hlist(hlist, v, hlist, i);
+            }
+            v++;
+        }
+    }
+    hlist.num = v;
+
     return hlist;
 }
 
@@ -655,25 +682,20 @@ int
 nmz_reverse_hlist(NmzResult hlist)
 {
     int m, n;
-    NmzResult tmp;
+    struct nmz_data tmp;
 
-    tmp.num  = 0;
-    tmp.data = NULL;
-    tmp.stat = SUCCESS;
-
-    nmz_malloc_hlist(&tmp, 1);
-    if (tmp.stat == ERR_FATAL)
-        return FAILURE;
     m = 0;
     n = hlist.num - 1;
     while (m < n) {
-	nmz_copy_hlist(tmp, 0, hlist, m);
-	nmz_copy_hlist(hlist, m, hlist, n);
-	nmz_copy_hlist(hlist, n, tmp, 0);
-	m++;
-	n--;
+        /* swap */
+        tmp = hlist.data[m];
+        hlist.data[m] = hlist.data[n];
+        hlist.data[n] = tmp;
+
+        m++;
+        n--;
     }
-    nmz_free_hlist(tmp);
+
     return 0;
 }
 
