@@ -2,10 +2,10 @@
  * 
  * libnamazu.c - Namazu library api
  *
- * $Id: libnamazu.c,v 1.35.8.9 2003/07/26 08:25:02 knok Exp $
+ * $Id: libnamazu.c,v 1.35.8.11 2008-03-06 15:38:09 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2003 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * Copyright (C) 1999 NOKUBI Takatsugu All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
@@ -27,15 +27,6 @@
  * 
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <sys/stat.h>
-#include <signal.h>
-#include <stdarg.h>
-
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -43,6 +34,18 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+
+#include <sys/types.h>
+#include <fcntl.h>
+#include <ctype.h>
+#include <sys/stat.h>
+#include <signal.h>
+#include <stdarg.h>
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -52,9 +55,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -207,7 +210,7 @@ nmz_is_regex_searchmode(void)
 char *
 nmz_msg(const char *fmt, ...)
 {
-    static char msg[BUFSIZE];
+    static char msg[BUFSIZE] = "";
     va_list args;
     
     va_start(args, fmt);
@@ -241,5 +244,3 @@ nmz_get_dyingmsg(void)
 {
     return dyingmsg;
 }
-
-
