@@ -2,10 +2,10 @@
  * 
  * re.c -
  * 
- * $Id: re.c,v 1.32.8.13 2006/09/19 15:21:58 opengl2772 Exp $
+ * $Id: re.c,v 1.32.8.15 2008-03-06 15:38:09 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2006 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,7 @@
  * This file must be encoded in EUC-JP encoding
  *
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -36,10 +34,19 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
