@@ -2,10 +2,10 @@
  * 
  * score.c - scoring-related functions of Namazu
  * 
- * $Id: score.c,v 1.3.4.8 2004/03/20 15:25:10 usu Exp $
+ * $Id: score.c,v 1.3.4.12 2008-03-06 15:38:09 opengl2772 Exp $
  * 
  * Copyright (C) 2001 Hajime BABA  All rights reserved.
- * Copyright (C) 2001,2003,2004 Namazu Project All rights reserved.
+ * Copyright (C) 2001-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -29,11 +29,6 @@
 
 /* #define SCORE_C_DEBUG    / * for internal use only */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <errno.h>
-#include <time.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -41,6 +36,17 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#include <math.h>
+#include <time.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -50,9 +56,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -308,11 +314,11 @@ get_freshness_factor(int date)
  *
  * factor  URI
  * ==================================
- * 2.0     http://www.foo.jp/
- * 1.7     http://www.foo.jp/a/
- * 1.5     http://www.foo.jp/a/b/
- * 1.2     http://www.foo.jp/a/b/c/
- * 1.0     http://www.foo.jp/a/b/c/d/
+ * 2.0     http://www.example.jp/
+ * 1.7     http://www.example.jp/a/
+ * 1.5     http://www.example.jp/a/b/
+ * 1.2     http://www.example.jp/a/b/c/
+ * 1.0     http://www.example.jp/a/b/c/d/
  *
  * NOTE: This factor function is derived empirically.
  *
@@ -431,18 +437,18 @@ nmz_recompute_score(NmzResult *hlist)
     double qds_dis_ratio = 1.0;
     int i;
     
-    orig = malloc(sizeof(int)*(hlist->num + 1));
+    orig = malloc(sizeof(int) * hlist->num);
     if (orig == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
 	 return;
     }
-    qds =  malloc(sizeof(double)*(hlist->num + 1));
+    qds =  malloc(sizeof(double) * hlist->num);
     if (qds == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
          free(orig);
 	 return;
     }
-    dis =  malloc(sizeof(double)*(hlist->num + 1));
+    dis =  malloc(sizeof(double) * hlist->num);
     if (dis == NULL) {
 	 nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
          free(orig);
