@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: util.c,v 1.75.4.17 2006/04/15 17:44:18 opengl2772 Exp $
+ * $Id: util.c,v 1.75.4.20 2008-07-10 19:46:45 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2006 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -27,16 +27,6 @@
  *
  */
 
-#include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <stdarg.h>
-#include <errno.h>
-#ifdef __EMX__
-#include <sys/types.h>
-#endif
-#include <sys/stat.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -44,10 +34,27 @@
 #  include "support.h"
 #endif
 
+#include <assert.h>
+#include <stdio.h>
+#ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+#endif
+#include <ctype.h>
+#include <stdarg.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
+#ifdef __EMX__
+#include <sys/types.h>
+#endif
+#include <sys/stat.h>
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -62,22 +69,22 @@
  *
  */
 
-static void reverse_byte_order (void*, int, int);
+static void reverse_byte_order (void*, size_t, size_t);
 static char decode_uri_sub(char c1, char c2);
 
 /* 
  * Reverse byte order. It's type independent.
  */
 static void 
-reverse_byte_order (void *p, int n, int size)
+reverse_byte_order (void *p, size_t n, size_t size)
 {
     int i, j;
     char *pp, tmp;
 
     pp = (char *)p;
-    for (i = 0; i < n; i++) {
+    for (i = 0; i < (int)n; i++) {
         char *c = (pp + (i * size));
-        for (j = 0; j < (size / 2); j++) {
+        for (j = 0; j < (int)(size / 2); j++) {
             tmp = *(c + j);
             *(c + j)= *(c + size - 1 - j);
             *(c + size - 1 - j) = tmp;
@@ -90,7 +97,7 @@ decode_uri_sub(char c1, char c2)
 {
     char c;
 
-    c =  ((c1 >= 'A' ? (_nmz_toupper((unsigned char)c1) - 'A') + 10 : (c1 - '0'))) * 16;
+    c = ((c1 >= 'A' ? (_nmz_toupper((unsigned char)c1) - 'A') + 10 : (c1 - '0'))) * 16;
     c += ( c2 >= 'A' ? (_nmz_toupper((unsigned char)c2) - 'A') + 10 : (c2 - '0'));
     return c;
 }
@@ -112,7 +119,7 @@ nmz_scan_oct(const char *start, int len, int *retlen)
 	retval |= *s++ - '0';
     }
 
-    *retlen = s - start;
+    *retlen = (int)(s - start);
     return retval;
 }
 
@@ -129,7 +136,7 @@ nmz_scan_hex(const char *start, int len, int *retlen)
 	retval |= (tmp - hexdigit) & 15;
 	s++;
     }
-    *retlen = s - start;
+    *retlen = (int)(s - start);
     return retval;
 }
 
@@ -336,7 +343,7 @@ nmz_pathcat(const char *base, char *name)
     win32 = 1;
 #endif
 
-    for (i = strlen(name) - 1; i >= 0; i--) {
+    for (i = (int)strlen(name) - 1; i >= 0; i--) {
         if (name[i] == '/' || (win32 && name[i] == '\\')) {
             strcpy(name, name + i + 1);
             break;
@@ -371,22 +378,22 @@ nmz_isnumstr(const char *str)
  * Substitute for tolower(3).
  */
 
-int 
+int
 _nmz_tolower(int c)
 {
     if (c >= 'A' && c <= 'Z') {
-	c = 'a' + c - 'A';
-	return c;
+        c = 'a' + c - 'A';
+        return c;
     }
     return c;
 }
 
-int 
+int
 _nmz_toupper(int c)
 {
     if (c >= 'a' && c <= 'z') {
-	c = 'A' + c - 'a';
-	return c;
+        c = 'A' + c - 'a';
+        return c;
     }
     return c;
 }
@@ -404,7 +411,7 @@ nmz_strlower(char *str)
 int 
 nmz_strprefixcasecmp(const char *str1, const char *str2)
 {
-    int len1, len2;
+    size_t len1, len2;
 
     len1 = strlen(str1);
     len2 = strlen(str2);
@@ -419,7 +426,7 @@ nmz_strprefixcasecmp(const char *str1, const char *str2)
 int 
 nmz_strprefixcmp(const char *str1, const char *str2)
 {
-    int len1, len2;
+    size_t len1, len2;
 
     len1 = strlen(str1);
     len2 = strlen(str2);
@@ -434,7 +441,7 @@ nmz_strprefixcmp(const char *str1, const char *str2)
 int 
 nmz_strsuffixcmp(const char *str1, const char *str2)
 {
-    int len1, len2;
+    size_t len1, len2;
 
     len1 = strlen(str1);
     len2 = strlen(str2);
