@@ -2,10 +2,10 @@
  * 
  * wakati.c -
  * 
- * $Id: wakati.c,v 1.26.8.1 2001/09/02 07:13:37 rug Exp $
+ * $Id: wakati.c,v 1.26.8.6 2007-12-05 16:50:47 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000,2001,2003,2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,16 +26,17 @@
  * 
  */
 
-#include <stdio.h>
-#include <ctype.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include <stdio.h>
+#include <ctype.h>
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -82,7 +83,7 @@ iskatakana(const char *chr)
     c = (uchar *)chr;
 
     if ((*c == 0xa5) &&
-	(*(c + 1) >= 0xa0) && (*(c + 1) <= 0xff))
+	(*(c + 1) >= 0xa0)) /* 0xa0 <= *(c + 1) <= 0xff */
     {
 	return 1;
     } else if ((*c == 0xa1) && (*(c + 1) == 0xbc)) { /* choon */ 
@@ -101,7 +102,7 @@ ishiragana(const char *chr)
     c = (uchar *)chr;
 
     if ((*c == 0xa4) &&
-	(*(c + 1) >= 0xa0) && (*(c + 1) <= 0xff))
+	(*(c + 1) >= 0xa0)) /* 0xa0 <= *(c + 1) <= 0xff */
     {
 	return 1;
     } else if ((*c == 0xa1) && (*(c + 1) == 0xbc)) { /* choon */ 
@@ -127,7 +128,7 @@ nmz_wakati(char *key)
 
     nmz_debug_printf("wakati original: [%s].\n", key);
 
-    for (i = 0; i < strlen(key); ) {
+    for (i = 0; i < (int)strlen(key); ) {
         type = detect_char_type(key + i);
 	if (nmz_iseuc(*(key + i))) {
 	    key_leng = 0;
@@ -175,9 +176,13 @@ nmz_wakati(char *key)
             while(*(key + i) && !nmz_iseuc(*(key + i))) {
                 /* As an initial attempt always success, 
                    outer 'for loop' can avoid infinite loop */
+                if (*(key + i) == '\t') {
+                    nmz_chomp(buf);
+                }
                 strncat(buf, key + i, 1);
                 i++;
             }
+            nmz_chomp(buf);
             strcat(buf, "\t");
 	}
     }
