@@ -2,10 +2,10 @@
  * 
  * cgi.c -
  * 
- * $Id: cgi.c,v 1.61.4.16 2005/11/02 17:42:42 opengl2772 Exp $
+ * $Id: cgi.c,v 1.61.4.23 2008-07-15 15:02:25 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2005 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,15 @@
  * 
  * 
  */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <assert.h>
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -42,9 +44,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -121,11 +123,14 @@ validate_idxname(const char * idxname)
 #endif
 
     if (*idxname == '\0' || *idxname == '/' || (win32 && *idxname == '\\')) {
-        printf("%s text/html" CRLF CRLF, MSG_MIME_HEADER);
+        printf("%s %s; %s" CRLF CRLF, MSG_MIME_HEADER,
+            "text/html", _("charset=ISO-8859-1"));
+        printf("<html><body>\n");
 	puts_entitize(idxname);
 	printf(" : ");
         printf(_("Invalid idxname."));
-        exit(EXIT_FAILURE);
+        printf("\n</body></html>\n");
+        exit_nmz(EXIT_FAILURE);
     }
     while (*idxname) {
         if (nmz_strprefixcasecmp("../", idxname) == 0 ||
@@ -133,11 +138,14 @@ validate_idxname(const char * idxname)
             (win32 && nmz_strprefixcasecmp("..\\", idxname) == 0) || 
             (win32 && nmz_strprefixcasecmp("..." , idxname) == 0)) 
         {
-	    printf("%s text/html" CRLF CRLF, MSG_MIME_HEADER);
+            printf("%s %s; %s" CRLF CRLF, MSG_MIME_HEADER,
+                "text/html", _("charset=ISO-8859-1"));
+            printf("<html><body>\n");
 	    puts_entitize(idxname);
 	    printf(" : ");
             printf(_("Invalid idxname."));
-            exit(EXIT_FAILURE);
+            printf("\n</body></html>\n");
+            exit_nmz(EXIT_FAILURE);
         }
 	/* Skip until next '/' */
 	while (*idxname && *idxname != '/' && !(win32 && *idxname == '\\')) {
@@ -212,7 +220,7 @@ free_cgi_vars(struct cgivar *cv)
 static char *
 get_query_string(void) 
 {
-    int contlen;
+    size_t contlen;
     char *script_name    = "";
     char *query_string   = "";
 
@@ -249,7 +257,7 @@ get_cgi_vars(const char *querystring)
     struct cgivar *cv = NULL; /* SHOULD BE NULL for sentinel! */
 
     while (1) {
-	int len;
+	size_t len;
 	const char *tmp;
         char *pSemicolon;
 	char name[BUFSIZE];
@@ -293,7 +301,7 @@ get_cgi_vars(const char *querystring)
 
 	if (cv == NULL) {
 	    fprintf(stderr, "an error occurred at add_cgivar.\n");
-	    exit(EXIT_FAILURE);
+	    exit_nmz(EXIT_FAILURE);
 	}
 
 	if (*querystring == '\0') {
@@ -412,7 +420,7 @@ process_cgi_var_sort(char *value, struct cgiarg *ca)
 	nmz_set_sortmethod(SORT_BY_DATE);
 	nmz_set_sortorder(ASCENDING);
     } else if (nmz_strprefixcasecmp(value, "field:") == 0) {
-	int n;
+	size_t n;
 	char field[BUFSIZE];
 
 	value += strlen("field:");
@@ -438,7 +446,9 @@ process_cgi_var_max(char *value, struct cgiarg *ca)
 {
     int max;
 
-    sscanf(value, "%d", &max);
+    if (sscanf(value, "%d", &max) != 1) {
+        return;
+    }
 
     if (max < 0)
 	max = 0;
@@ -453,7 +463,10 @@ process_cgi_var_whence(char *value, struct cgiarg *ca)
 {
     int whence;
 
-    sscanf(value, "%d", &whence);
+    if (sscanf(value, "%d", &whence) != 1) {
+        return;
+    }
+
     if (whence < 0) {
 	whence = 0;
     }
@@ -537,7 +550,7 @@ init_cgi(char *query, char *subquery)
 
     if (process_cgi_vars(&ca)) {
 	show_mini_usage();   /* if it is NOT CGI, show usage and exit */
-	exit(EXIT_FAILURE);
+	exit_nmz(EXIT_FAILURE);
     }
 
     if (nmz_get_idxnum() == 0) {
