@@ -2,10 +2,10 @@
  * 
  * form.c -
  * 
- * $Id: form.c,v 1.60.4.14.2.2 2004/12/08 07:35:06 opengl2772 Exp $
+ * $Id: form.c,v 1.60.4.23 2008-07-15 14:29:10 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2004 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,10 +26,6 @@
  * 
  */
 
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -37,6 +33,14 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -46,9 +50,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -96,7 +100,7 @@ cmp_element(const char *s1, const char *s2)
             }
             s2++;
         }
-        if (toupper(*s1) != toupper(*s2)) {
+        if (_nmz_toupper((unsigned char)*s1) != _nmz_toupper((unsigned char)*s2)) {
             break;
         }
     }
@@ -108,44 +112,78 @@ cmp_element(const char *s1, const char *s2)
 }
 
 /* 
- * replace <input type="text" name="query"  value="hogehoge"> 
+ * replace <input type="text" name="query" value="hogehoge"> 
  */
 static enum nmz_stat
 replace_query_value(const char *p, const char *query)
 {
-    char *query_external = NULL;
-
     if (cmp_element(p, (char *)"input type=\"text\" name=\"query\"") == 0) {
-	char *converted = nmz_codeconv_external(query);
+	char *converted = nmz_query_external(query);
+
 	if (converted == NULL) {
 	    die("%s", strerror(errno));
 	}
 
-        query_external = converted;
-        while(*query_external == '\t') {
-            query_external++;
-        }
-
         for (; *p; p++)
             fputc(*p, stdout);
-        printf(" value=\"");
-
-        html_print(query_external);  /* for treating <>&" chars in the query. */
-        printf("\"");
 
+        {
+            printf(" value=\"");
+            html_print(converted);  /* for treating <>&" chars in the query. */
+            printf("\"");
+        }
 	free(converted);
         return SUCCESS;
     }
     return FAILURE;
 }
 
+char *
+nmz_query_external(const char *query)
+{
+    char *query_external;
+    char *alloc;
+    char *pos, *p;
+
+    alloc = strdup(query);
+    if (alloc == NULL) { /* */
+        nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
+        return NULL;
+    }
+
+    nmz_codeconv_internal(alloc);
+
+    p = alloc;
+    while(*p == '\t') {
+        p++;
+    }
+    pos = p;
+
+    /* tr/\x01-\x1f\x7f/ /; # Remove control char. */
+    while(*p != '\0') {
+        if ((*p >= 0x01 && *p <= 0x1f) || *p == 0x7f) {
+            *p = ' ';
+        }
+        p++;
+    }
+
+    query_external = nmz_codeconv_external(pos);
+    free(alloc);
+    if (query_external == NULL) {
+        nmz_set_dyingmsg(nmz_msg("%s", strerror(errno)));
+        return NULL;
+    }
+
+    return query_external;
+}
+
 /*
  * Delete string (case insensitive)
  */
 static void 
 delete_str(char *s, char *d)
 {
-    int l;
+    size_t l;
     char *tmp;
 
     l = strlen(d);
@@ -178,7 +216,6 @@ get_value(const char *s, char *value)
 static void 
 get_select_name(const char *s, char *value)
 {
-    *value = '\0';
     for (; *s; s++) {
         if (cmp_element(s, (char *)"select name=\"") == 0) {
             s = (char *)strchr(s, '"') + 1;
@@ -187,6 +224,9 @@ get_select_name(const char *s, char *value)
             }
             *value = '\0';
             return;
+        } else if (cmp_element(s, (char *)"/select") == 0) {
+            *value = '\0';
+            return;
         }
     }
 }
@@ -197,7 +237,7 @@ select_option(char *s, const char *name, const char *subquery)
     char value[BUFSIZE] = "";
 
     if (cmp_element(s, (char *)"option") == 0) {
-	if(strcmp(nmz_getenv("QUERY_STRING"), "")) {
+	if (strcmp(nmz_getenv("QUERY_STRING"), "")) {
             delete_str(s, (char *)"selected ");
 	}
         fputs(s, stdout);
@@ -224,9 +264,10 @@ select_option(char *s, const char *name, const char *subquery)
             } else if ((nmz_strprefixcasecmp(value, "field:") == 0) && 
 		       nmz_get_sortmethod() == SORT_BY_FIELD) 
 	    {
-		char *p;
-		int n, order = DESCENDING;
-		char field[BUFSIZE] = "";
+                char *p;
+                size_t n;
+                int order = DESCENDING;
+                char field[BUFSIZE] = "";
 
 		p = value + strlen("field:");
 		n = strspn(p, FIELD_SAFE_CHARS);
@@ -283,7 +324,7 @@ check_checkbox(char *str)
         char *pp;
         int db_count, searched;
 
-	if(strcmp(nmz_getenv("QUERY_STRING"), "")) {
+	if (strcmp(nmz_getenv("QUERY_STRING"), "")) {
             delete_str(str, (char *)"checked");
 	}
         fputs(str, stdout);
@@ -324,7 +365,7 @@ handle_tag(const char *start, const char *end, const char *query,
                char *select_name, const char *subquery)
 {
     char tmp[BUFSIZE] = "";
-    int l;
+    size_t l;
 
     l = end - start + 1;
     if (l < BUFSIZE - 1) {
@@ -349,6 +390,17 @@ read_headfoot(const char *fname)
     char *buf, *p, tmpfname[BUFSIZE] = "", suffix[BUFSIZE] = "";
     char *script_name;
     char *document_name;
+    size_t bufsize, newsize;
+    int i;
+
+    struct nmz_param {
+        char *name;
+        size_t len_name;
+        char *value;
+        size_t len_value;
+    };
+    struct nmz_param nmz_param_table[2];
+
 
     if (nmz_choose_msgfile_suffix(fname, suffix) != SUCCESS) {
 	nmz_warn_printf("%s: %s", fname, strerror(errno));
@@ -387,20 +439,60 @@ read_headfoot(const char *fname)
 	    if (*s == '?') {*s = 0; break;}
     }
 
-    /* Expand buf memory for replacing {cgi} and {doc} */
-    buf = (char *)realloc(buf, strlen(buf) + strlen(script_name) + strlen(document_name) + 2);
-    if (buf == NULL) {
-        return NULL;
+
+    /*
+       Namazu Parameter
+         0 ... {cgi}
+         1 ... {doc}
+     */
+    nmz_param_table[0].name = "{cgi}";
+    nmz_param_table[0].len_name = strlen(nmz_param_table[0].name);
+    nmz_param_table[0].value = script_name;
+    nmz_param_table[0].len_value = strlen(nmz_param_table[0].value);
+
+    nmz_param_table[1].name = "{doc}";
+    nmz_param_table[1].len_name = strlen(nmz_param_table[1].name);
+    nmz_param_table[1].value = document_name;
+    nmz_param_table[1].len_value = strlen(nmz_param_table[1].value);
+
+    newsize = bufsize = strlen(buf) + 1;
+    for (i = 0; i < sizeof(nmz_param_table) / sizeof(struct nmz_param); i++) {
+        for (p = buf; (p = strstr(p, nmz_param_table[i].name)) != NULL; p += nmz_param_table[i].len_name) {
+            newsize += (nmz_param_table[i].len_value - nmz_param_table[i].len_name);
+        }
+    }
+
+    if (newsize > bufsize) {
+        /* Expand buf memory for replacing {cgi} and {doc} */
+        buf = (char *)realloc(buf, newsize);
+        if (buf == NULL) {
+            return NULL;
+        }
     }
 
     /* Replace {cgi} with a proper namazu.cgi location */
-    while ((p = strstr(buf, "{cgi}")) != NULL) {
-	subst(p, "{cgi}", script_name);
-    }
-
     /* Replace {doc} with the name of the calling document eg, using SSI */
-    while ((p = strstr(buf, "{doc}")) != NULL) {
-	subst(p, "{doc}", document_name);
+    p = buf;
+    while (1) {
+        char *pp = NULL;
+        int idx = 0;
+
+        for (i = 0; i < sizeof(nmz_param_table) / sizeof(struct nmz_param); i++) {
+            char *pos;
+
+            if ((pos = strstr(p, nmz_param_table[i].name)) != NULL) {
+                if (pp == NULL || pp > pos) {
+                    pp = pos;
+                    idx = i;
+                }
+            }
+        }
+        if (pp == NULL) {
+            break;
+        }
+
+        subst(pp, nmz_param_table[idx].name, nmz_param_table[idx].value);
+        p = pp + nmz_param_table[idx].len_value;
     }
 
     return buf;
@@ -454,16 +546,12 @@ print_headfoot(const char * fname, const char * query, const char *subquery)
         if (f == 0 && *p == '<') {
             if (nmz_strprefixcasecmp(p, "</title>") == 0) {
 		if (*query != '\0') {
-		    char *converted = nmz_codeconv_external(query);
+		    char *converted = nmz_query_external(query);
                     if (converted) {
-                        char *query_external = converted;
-                        while(*query_external == '\t') {
-                            query_external++;
-                        }
-		        printf(": &lt;");
-		        html_print(query_external);
-		        printf("&gt;");
-		        free(converted);
+                        printf(": &lt;");
+                        html_print(converted);
+                        printf("&gt;");
+                        free(converted);
                     }
 		}
 		printf("</title>\n");
@@ -498,7 +586,7 @@ print_headfoot(const char * fname, const char * query, const char *subquery)
 		fputs(">", stdout);
 	    } else {            /* for XHTML */
 		handle_tag(p + 1, q - 2, query, name, subquery);
-		fputs("/>", stdout);
+		fputs(" />", stdout);
 	    }
             p = q;
         } else {
@@ -517,8 +605,3 @@ print_headfoot(const char * fname, const char * query, const char *subquery)
     }
     free(buf);
 }
-
-
-
-
-
