@@ -1,11 +1,11 @@
 /*
  * 
- * namazu.c - search client of Namazu
+ * namazu-cgi.c - search client of Namazu
  *
- * $Id: namazu-cgi.c,v 1.16.8.10 2006/08/04 11:59:01 opengl2772 Exp $
+ * $Id: namazu-cgi.c,v 1.16.8.15 2008-07-19 17:56:42 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2006 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,10 @@
  * 
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -43,10 +47,6 @@
 #define	SIGALRM	14	/* alarm clock */
 #endif
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #else
@@ -56,9 +56,9 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -76,6 +76,13 @@
 #include "message.h"
 #include "system.h"
 #include "result.h"
+#include "charset.h"
+
+/*
+ * Extern variables.
+ */
+
+NMZ_HANDLE handle_charset = (NMZ_HANDLE)0;
 
 /*
  *
@@ -108,7 +115,7 @@ combine_pathname(char *dest, const char *command, const char *name)
 #endif
 
     strcpy(dest, command);
-    for (i = strlen(dest) - 1; i >= 0; i--) {
+    for (i = (int)strlen(dest) - 1; i >= 0; i--) {
 	if (dest[i] == '/' || (win32 && dest[i] == '\\')) {
 	    break;
 	}
@@ -130,6 +137,13 @@ main(int argc, char **argv)
     char query[BUFSIZE] = "", subquery[BUFSIZE] = "";
     char *localedir = getenv("NAMAZULOCALEDIR");
 
+    /*
+     *
+     * create resource.
+     *
+     */
+    handle_charset = create_charset_list();
+
     /*
      * To support a binary package for Windows, we should
      * allow to change LOCALEDIR with the environment
@@ -203,5 +217,12 @@ main(int argc, char **argv)
         die(nmz_get_dyingmsg());
     }
 
-    return 0;
+    /*
+     *
+     * free resource.
+     *
+     */
+    nmz_free_handle(handle_charset);
+
+    return EXIT_SUCCESS;
 }
