@@ -1,8 +1,8 @@
 /*
- * $Id: output.c,v 1.80.4.25 2006/02/05 18:53:30 opengl2772 Exp $
+ * $Id: output.c,v 1.80.4.39 2008-07-19 17:56:42 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2004 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2008 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -22,11 +22,7 @@
  * 
  * 
  */
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <stdarg.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -34,10 +30,19 @@
 #  include "support.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <stdarg.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -55,6 +60,7 @@
 #include "idxname.h"
 #include "query.h"
 #include "system.h"
+#include "charset.h"
 
 static int htmlmode    = 0;
 static int cgimode     = 0;
@@ -78,7 +84,13 @@ static char template_suffix[BUFSIZE] = "normal"; /* suffix of NMZ.result.* */
  */ 
 static char emphasis_start_tag[BUFSIZE] = "<strong class=\"keyword\">";
 static char emphasis_end_tag[BUFSIZE]   = "</strong>";
-char contenttype[BUFSIZE] = "text/html";
+static char contenttype[BUFSIZE] = "text/html";
+
+/*
+ * Extern variables.
+ */
+
+extern NMZ_HANDLE handle_charset;
 
 /*
  *
@@ -117,11 +129,14 @@ static void
 emprint(char *s, int entity_encode)
 {
     int i;
+    int nestedtags = 0;
     for (i = 0; i < BUFSIZE * 16 && *s; s++) {
 	if (*s == EM_START_MARK) {
+            nestedtags++;
 	    fputs(emphasis_start_tag, stdout);
 	    continue;
 	} else if (*s == EM_END_MARK) {
+            if (nestedtags > 0) nestedtags--;
 	    fputs(emphasis_end_tag, stdout);
 	    continue;
 	} 
@@ -135,6 +150,12 @@ emprint(char *s, int entity_encode)
 		fputs("&amp;", stdout);
 	    } else if (*s == '"') {
 		fputs("&quot;", stdout);
+            } else if (*s == '\'') {
+                fputs("&#39;", stdout);
+            } else if (*s == '(') {
+                fputs("&#40;", stdout);
+            } else if (*s == ')') {
+                fputs("&#41;", stdout);
 	    } else {
 		fputc(*s, stdout);
 	    }
@@ -142,6 +163,9 @@ emprint(char *s, int entity_encode)
 	    fputc(*s, stdout);
 	}
     }
+    for (; nestedtags > 0; nestedtags--) {
+       fputs(emphasis_end_tag, stdout);
+    }
 }
 
 /*
@@ -149,9 +173,10 @@ emprint(char *s, int entity_encode)
  */
 static void
 unhtml_buffer(char *ostr) {
-    int f, i;
+    int f;
+    size_t i;
     char buf[BUFSIZE] = "", *str;
-    int len;
+    size_t len;
     len = strlen(ostr) + 1;
     str = ostr;
     for (f = 0, i = 0; i < BUFSIZE && *str; str++) {
@@ -171,7 +196,7 @@ unhtml_buffer(char *ostr) {
 		return; 
 	    }
 	    if (*(p + 2) == 'J' || *(p + 2) == 'B' || *(p + 2) == 'H') {
-		int len = p - str + 3;
+		size_t len = p - str + 3;
 		strncpy(buf + i, str, len);
 		i += len;
 		str += len;
@@ -181,15 +206,15 @@ unhtml_buffer(char *ostr) {
 	    }
 	}
 
-	if (strncasecmp(str, "<br>", 4) == 0 && *(str + 4) != '\n') {
-	    buf[i++] = '\n';
-	    str += 3;
-	    continue;
-	} else if (strncasecmp(str, "<br />", 6) == 0 && *(str + 6) != '\n') {
-	    buf[i++] = '\n';
-	    str += 5;
-	    continue;
-	}
+        if (strncasecmp(str, "<br>", 4) == 0 && *(str + 4) != '\n') {
+            buf[i++] = '\n';
+            str += 3;
+            continue;
+        } else if (strncasecmp(str, "<br />", 6) == 0 && *(str + 6) != '\n') {
+            buf[i++] = '\n';
+            str += 5;
+            continue;
+        }
 	if (*str == '<') {
 	    f = 1;
 	    continue;
@@ -220,6 +245,7 @@ unhtml_buffer(char *ostr) {
     strncpy(ostr, buf, len);
     ostr[len - 1] = '\0';
 }
+
 static void 
 fputs_without_html_tag(const char *str, FILE *fp)
 {
@@ -496,7 +522,7 @@ print_query(const char * qs, int w)
 	if (strncmp(qs, "whence=", strlen("whence=")) == 0) {
 	    foo = 1;
 	    printf("whence=%d", w);
-	    for (qs += strlen("whence="); isdigit((unsigned char)*qs); qs++);
+	    for (qs += strlen("whence="); nmz_isdigit((unsigned char)*qs); qs++);
 	} else {
 	    /* '"' is converted to entities "&quot;" */
 	    putc_entitize(*qs);
@@ -721,12 +747,10 @@ print_errmsg(int errid)
 enum nmz_stat 
 print_result(NmzResult hlist, const char *query, const char *subquery)
 {
-
-    if (is_htmlmode() && is_cgimode()) {
-	printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, contenttype);
-    }
-
     if (is_htmlmode()) {
+        if (is_cgimode()) {
+            printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, get_contenttype());
+        }
 	print_headfoot(NMZ.head, query, subquery);
     }
 
@@ -824,7 +848,7 @@ void
 print_default_page (void) {
     if (is_htmlmode()) {
         if (is_cgimode()) {
-	    printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, contenttype);
+	    printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, get_contenttype());
         }
 	print_headfoot(NMZ.head, "", "");
 	print_msgfile(NMZ.body);
@@ -855,8 +879,14 @@ putc_entitize(int c)
 	    fputs("&amp;", stdout);
 	} else if (c == '"') {
 	    fputs("&quot;", stdout);
-	} else if (c == '\'') {
-	    fputs("&apos;", stdout);
+        } else if (c == '\'') {
+            /* not support &apos; for HTML 4.01 */
+            /* fputs("&apos;", stdout); */
+            fputs("&#39;", stdout);
+        } else if (c == '(') {
+            fputs("&#40;", stdout);
+        } else if (c == ')') {
+            fputs("&#41;", stdout);
 	} else {
 	    fputc(c, stdout);
 	}
@@ -887,6 +917,78 @@ set_contenttype(const char *str)
     strncpy(contenttype, str, BUFSIZE - 1);
 }
 
+char *
+get_contenttype(void)
+{
+    static char buff[BUFSIZE];
+    char *p;
+    int is_charset = 0;
+
+
+    buff[0] = '\0';
+    p = strtok(contenttype, " \t\n\r;");
+    while(p) {
+        if (!strncasecmp(p, "charset=", strlen("charset="))) {
+            is_charset = 1;
+            break;
+        }
+        p = strtok(NULL, " \t\n\r;");
+    }
+
+    if (!is_charset) {
+        char *charset = "ISO-8859-1";
+        int cont = 1;
+        char suffix[BUFSIZE] = "";
+        char *lang;
+
+        buff[0] = '\0';
+        if (nmz_choose_msgfile_suffix(NMZ.head, suffix) == SUCCESS) {
+            if (suffix[0] == '.') {
+                strncpy(buff, suffix + 1, BUFSIZE - 1);
+            } else {
+                buff[0] = '\0';
+            }
+        }
+        lang = nmz_get_lang();
+        if (!strncasecmp(lang, "ja", 2)) {
+            if (!strcasecmp(lang, "ja_JP.SJIS")) {
+                /* Shift_JIS */
+                strncpy(buff, "ja_JP.SJIS", BUFSIZE - 1);
+            } else if (!strcasecmp(lang, "ja_JP.ISO-2022-JP")) {
+                /* ISO-2022-JP */
+                strncpy(buff, "ja_JP.ISO-2022-JP", BUFSIZE - 1);
+            } else {
+                /* EUC-JP */
+                strncpy(buff, "ja", BUFSIZE - 1);
+            }
+        }
+
+        while(cont == 1) {
+            int i;
+            char *value;
+            cont = 0;
+
+            if ((value = nmz_find_first_strlist(handle_charset, buff))) {
+                charset = value;
+                break;
+            }
+
+            for(i = (int)strlen(buff) - 1; i >= 0; i--) {
+                if (buff[i] == '.' || buff[i] == '_' || buff[i] == '@') {
+                    buff[i] = '\0';
+                    cont = 1;
+                    break;
+                }
+            }
+        }
+
+        sprintf(buff, "%s; charset=%s", contenttype, charset);
+        return(buff);
+    }
+
+    return(contenttype);
+}
+
 void 
 set_htmlmode(int mode)
 {
@@ -1021,7 +1123,7 @@ get_templatesuffix(void)
 void 
 html_print(const char *str)
 {
-    char buf[BUFSIZE * 16];
+    char buf[BUFSIZE * 16] = "";
     int is_nmz_html = 0;
 
     if ((int)*str == (int)'\t') { /* Namazu's HTML message */
@@ -1061,26 +1163,32 @@ die(const char *fmt, ...)
     fflush(stdout);
     fflush(stderr);
 
-    if (is_cgimode()) {
-	printf("%s %s" CRLF CRLF, MSG_MIME_HEADER, "text/html");
-	printf(_("<h2>Error</h2>\n<p>"));
-	va_start(args, fmt);
-	vprintf(fmt, args);
-	va_end(args);
-	printf(_("</p>"));
-	if (fmt[strlen(fmt) - 1] != '\n') {
-	    printf("\n");
-	}
+    if (is_htmlmode()) {
+        static char msg[BUFSIZE] = "";
+
+        va_start(args, fmt);
+        vsnprintf(msg, BUFSIZE - 1, fmt, args);
+        va_end(args);
+
+        if (is_cgimode()) {
+            printf("%s %s; %s" CRLF CRLF, MSG_MIME_HEADER,
+                "text/html", _("charset=ISO-8859-1"));
+        }
+        printf("<html><body>\n");
+        printf(_("<h2>Error</h2>\n<p>"));
+        puts_entitize(msg);
+        printf(_("</p>"));
+        printf("\n</body></html>\n");
     } else {
 	fprintf(stderr, "%s: ", PACKAGE);
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
 	va_end(args);
-       if (fmt[strlen(fmt) - 1] != '\n') {
-	   fprintf(stderr, "\n");
-       }
+        if (fmt[strlen(fmt) - 1] != '\n') {
+            fprintf(stderr, "\n");
+        }
     }
 
-    exit(EXIT_FAILURE);
+    exit_nmz(EXIT_FAILURE);
 }
 
