@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: rcfile.c,v 1.31.4.14 2007/02/25 03:34:53 opengl2772 Exp $
+ * $Id: rcfile.c,v 1.31.4.20 2009-02-16 17:43:43 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000-2007 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2009 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,6 @@
  * 
  */
 
-#include <assert.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -34,10 +30,18 @@
 #  include "support.h"
 #endif
 
+#include <assert.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "namazu.h"
@@ -55,6 +59,13 @@
 #include "idxname.h"
 #include "output.h"
 #include "score.h"
+#include "charset.h"
+
+/*
+ * Extern variables.
+ */
+
+extern NMZ_HANDLE handle_charset;
 
 /*
  * Default directory to place namazurc.
@@ -109,7 +120,7 @@ struct _StrList {
  */
 
 static char *getenv_namazurc ( void );
-static int get_rc_arg ( const char *line, char *arg );
+static size_t get_rc_arg ( const char *line, char *arg );
 static void replace_home ( char *str );
 static StrList * add_strlist(StrList *list, const char *arg);
 static void free_strlist(StrList *list);
@@ -137,6 +148,7 @@ static enum nmz_stat process_rc_maxmatch ( const char *directive, const StrList
 static enum nmz_stat process_rc_contenttype ( const char *directive, const StrList *args );
 static enum nmz_stat process_rc_suicidetime ( const char *directive, const StrList *args );
 static enum nmz_stat process_rc_regexsearch ( const char *directive, const StrList *args );
+static enum nmz_stat process_rc_charset ( const char *directive, const StrList *args );
 
 struct conf_directive {
     char *name;
@@ -165,6 +177,7 @@ static struct conf_directive directive_tab[] = {
     { "CONTENTTYPE",   1, 0, process_rc_contenttype },
     { "SUICIDE_TIME",  1, 0, process_rc_suicidetime },
     { "REGEX_SEARCH",  1, 0, process_rc_regexsearch },
+    { "CHARSET",       2, 0, process_rc_charset },
     { NULL,            0, 0, NULL }
 };
 
@@ -190,9 +203,9 @@ process_rc_debug(const char *directive, const StrList *args)
     char *arg1 = args->value;
 
     if (strcasecmp(arg1, "ON") == 0) {
-	nmz_set_debugmode(1);
+        nmz_set_debugmode(1);
     } else if (strcasecmp(arg1, "OFF") == 0) {
-	nmz_set_debugmode(0);
+        nmz_set_debugmode(0);
     }
     return SUCCESS;
 }
@@ -216,7 +229,7 @@ process_rc_alias(const char *directive, const StrList *args)
     char *arg2 = args->next->value;
 
     if (nmz_add_alias(arg1, arg2) != SUCCESS) {
-	return FAILURE;
+        return FAILURE;
     }
     return SUCCESS;
 }
@@ -228,7 +241,7 @@ process_rc_replace(const char *directive, const StrList *args)
     char *arg2 = args->next->value;
 
     if (nmz_add_replace(arg1, arg2) != SUCCESS) {
-	return FAILURE;
+        return FAILURE;
     }
     return SUCCESS;
 }
@@ -239,9 +252,9 @@ process_rc_logging(const char *directive, const StrList *args)
     char *arg1 = args->value;
 
     if (strcasecmp(arg1, "ON") == 0) {
-	nmz_set_loggingmode(1);
+        nmz_set_loggingmode(1);
     } else if (strcasecmp(arg1, "OFF") == 0) {
-	nmz_set_loggingmode(0);
+        nmz_set_loggingmode(0);
     }
 
     return SUCCESS;
@@ -292,7 +305,7 @@ process_rc_lang(const char *directive, const StrList *args)
      * It works if environment variable LANG is not set.
      */
     if (getenv("LANG") == NULL) {
-	nmz_set_lang(arg1);
+        nmz_set_lang(arg1);
     }
     return SUCCESS;
 }
@@ -359,14 +372,26 @@ process_rc_regexsearch(const char *directive, const StrList *args)
     char *arg1 = args->value;
 
     if (strcasecmp(arg1, "ON") == 0) {
-	nmz_set_regex_searchmode(1);
+        nmz_set_regex_searchmode(1);
     } else if (strcasecmp(arg1, "OFF") == 0) {
-	nmz_set_regex_searchmode(0);
+        nmz_set_regex_searchmode(0);
     }
 
     return SUCCESS;
 }
 
+static enum nmz_stat
+process_rc_charset(const char *directive, const StrList *args)
+{
+    char *arg1 = args->value;
+    char *arg2 = args->next->value;
+
+    if (nmz_add_strlist(handle_charset, arg1, arg2) != SUCCESS) {
+        return FAILURE;
+    }
+    return SUCCESS;
+}
+
 /* 
  * Get the environment variable of NAMAZURC, NAMAZUCONF or
  * NAMAZUCONFPATH.  and return it. Original of this code is
@@ -379,62 +404,62 @@ getenv_namazurc(void)
 
     env_namazurc = getenv("NAMAZURC");
     if (env_namazurc != NULL) {
-	return env_namazurc;
+        return env_namazurc;
     }
 
     /* For backward compatibility. */
     env_namazurc = getenv("NAMAZUCONF");
     if (env_namazurc != NULL) {
-	return env_namazurc;
+        return env_namazurc;
     }
 
     /* For backward compatibility. */
     env_namazurc = getenv("NAMAZUCONFPATH");
     if (env_namazurc != NULL) {
-	return env_namazurc;
+        return env_namazurc;
     }
 
     return NULL;
 }
 
-static int 
+static size_t 
 get_rc_arg(const char *line, char *arg)
 {
     arg[BUFSIZE - 1] = '\0';
     *arg = '\0';
     if (*line != '"') {
-	size_t n = strcspn(line, " \t");
-	if (n >= BUFSIZE) n = BUFSIZE - 1;
-	strncpy(arg, line, n);
-	arg[n] = '\0';     /* Hey, don't forget this after strncpy()! */
-	return n;
+        size_t n = strcspn(line, " \t");
+        if (n >= BUFSIZE) n = BUFSIZE - 1;
+        strncpy(arg, line, n);
+        arg[n] = '\0';     /* Hey, don't forget this after strncpy()! */
+        return n;
     } else {  /* double quoted argument */
-	int n;
-	line++;
-	n = 1;
-	do {
-	    size_t nn = strcspn(line, "\"\\");
-	    if (nn >= (BUFSIZE - strlen(arg))) nn = BUFSIZE - strlen(arg) - 1;
-	    strncat(arg, line, nn);
-	    n += nn;
-	    line += nn;
-	    arg[n] = '\0';
-	    if (n >= BUFSIZE) return n;
-	    if (*line == '\0') {  /* terminator not matched */
-		errmsg = _("can't find string terminator");
-		return 0;
-	    }
-	    if  (*line == '"') {  /* ending */
-		n++;
-		return n;
-	    }
-	    if (*line == '\\') {  /* escaping character */
-		strncat(arg, line + 1, 1);
-		n += 2;
-		line += 2;
-	    }
-	} while (1);
-	return n;
+        size_t n;
+        line++;
+        n = 1;
+        do {
+            size_t nn = strcspn(line, "\"\\");
+            if (nn >= (BUFSIZE - strlen(arg))) nn = BUFSIZE - strlen(arg) - 1;
+            strncat(arg, line, nn);
+            n += nn;
+            line += nn;
+            arg[n] = '\0';
+            if (n >= BUFSIZE) return n;
+            if (*line == '\0') {  /* terminator not matched */
+                errmsg = _("can't find string terminator");
+                return 0;
+            }
+            if  (*line == '"') {  /* ending */
+                n++;
+                return n;
+            }
+            if (*line == '\\') {  /* escaping character */
+                strncat(arg, line + 1, 1);
+                n += 2;
+                line += 2;
+            }
+        } while (1);
+        return n;
     }
 }
 
@@ -449,15 +474,15 @@ replace_home(char *str)
 
     strcpy(tmp, str);
     if (nmz_strprefixcmp(tmp, "~/") == 0) {
-	char *home;
-	/* Checke a home directory */
-	if ((home = getenv("HOME")) != NULL) {
-	    strncpy(tmp, home, BUFSIZE - 1);
-	    strncat(tmp, "/", BUFSIZE - strlen(tmp) - 1);
-	    strncat(tmp, str + strlen("~/"), BUFSIZE - strlen(tmp) - 1);
-	    strncpy(str, tmp, BUFSIZE - 1);
-	    return;
-	}
+        char *home;
+        /* Checke a home directory */
+        if ((home = getenv("HOME")) != NULL) {
+            strncpy(tmp, home, BUFSIZE - 1);
+            strncat(tmp, "/", BUFSIZE - strlen(tmp) - 1);
+            strncat(tmp, str + strlen("~/"), BUFSIZE - strlen(tmp) - 1);
+            strncpy(str, tmp, BUFSIZE - 1);
+            return;
+        }
     }
 
     return;
@@ -473,28 +498,28 @@ add_strlist(StrList *list, const char *arg)
 
     newp = malloc(sizeof(StrList));
     if (newp == NULL) {
-	return NULL;
+        return NULL;
     }
 
     newp->value = malloc(strlen(arg) + 1);
     if (newp->value == NULL) {
         free(newp);
-	return NULL;
+        return NULL;
     }
     strcpy(newp->value, arg);
     newp->next = NULL;
 
     if (list == NULL) {
-	return newp;
+        return newp;
     } else {
-	StrList *ptr = list;
+        StrList *ptr = list;
 
-	while (ptr->next != NULL) {
-	    ptr = ptr->next;
-	}
-	assert(ptr->next == NULL);
-	ptr->next = newp;
-	return list;
+        while (ptr->next != NULL) {
+            ptr = ptr->next;
+        }
+        assert(ptr->next == NULL);
+        ptr->next = newp;
+        return list;
     }
     assert(0);
     /* NOTREACHED */
@@ -506,9 +531,9 @@ free_strlist(StrList *list)
     StrList *next, *ptr;
 
     for (ptr = list; ptr != NULL; ptr = next) {
-	next = ptr->next;
-	free(ptr->value);
-	free(ptr);
+        next = ptr->next;
+        free(ptr->value);
+        free(ptr);
     }
 }
 
@@ -522,65 +547,65 @@ static StrList *
 get_rc_args(const char *line)
 {
     StrList *list = NULL;
-    int n;
+    size_t n;
 
     /* Skip white spaces in the beginning of this line */
     n = strspn(line, " \t");    /* skip white spaces */
     line += n;
     /* Determine whether or not this line is only a blank */
     if (*line == '\0') {
-	return add_strlist(list, "BLANK");
+        return add_strlist(list, "BLANK");
     }
 
     /* Determine whether or not this line is only a comment */
     if (*line == '#') {
-	return add_strlist(list, "COMMENT");
+        return add_strlist(list, "COMMENT");
     }
 
     /* Get a directive name. */
     {
-	char directive[BUFSIZE] = "";
-	n = strspn(line, DIRECTIVE_CHARS);
-	if (n == 0) {
-	    errmsg = _("invalid directive name");
-	    return 0;
-	}
-	if (n >= BUFSIZE) n = BUFSIZE - 1;
-	strncpy(directive, line, n);
-	directive[n] = '\0';  /* Hey, don't forget this after strncpy()! */
-	list = add_strlist(list, directive);
-	line += n;
+        char directive[BUFSIZE] = "";
+        n = strspn(line, DIRECTIVE_CHARS);
+        if (n == 0) {
+            errmsg = _("invalid directive name");
+            return 0;
+        }
+        if (n >= BUFSIZE) n = BUFSIZE - 1;
+        strncpy(directive, line, n);
+        directive[n] = '\0';  /* Hey, don't forget this after strncpy()! */
+        list = add_strlist(list, directive);
+        line += n;
     }
 
     /* Skip a delimiter after a directive */
     n = strspn(line, " \t");    /* skip white spaces */
     line += n;
     if (n == 0) {
-	errmsg = _("can't find arguments");
-	return 0;
+        errmsg = _("can't find arguments");
+        return 0;
     }
 
     while (1) {
-	char arg[BUFSIZE] = "";
-	n = get_rc_arg(line, arg);
-	if (n == 0) { /* cannot get arg1 */
-	    return NULL;
-	}
-	line += n;
+        char arg[BUFSIZE] = "";
+        n = get_rc_arg(line, arg);
+        if (n == 0) { /* cannot get arg1 */
+            return NULL;
+        }
+        line += n;
 
-	/* Replace ~/ */
-	replace_home(arg);
+        /* Replace ~/ */
+        replace_home(arg);
 
-	list = add_strlist(list, arg);
+        list = add_strlist(list, arg);
 
-	/* Skip a delimiter after the arg */
-	n = strspn(line, " \t");    /* skip white spaces */
-	line += n;
+        /* Skip a delimiter after the arg */
+        n = strspn(line, " \t");    /* skip white spaces */
+        line += n;
 
-	if (*line == '\0' || *line == '#') {  /* allow comment at the ending */
-	    /* This line has only one argument (arg) */
-	    break;
-	}
+        if (*line == '\0' || *line == '#') {  /* allow comment at the ending */
+            /* This line has only one argument (arg) */
+            break;
+        }
     }
 
     return list;
@@ -599,7 +624,7 @@ parse_rcfile(const char *line, int lineno)
     */
     d_args = get_rc_args(line); 
     if (d_args == NULL) { 
-	return FAILURE; /* error */
+        return FAILURE; /* error */
     }
     directive = d_args->value; /* the first arg is a directive. */
     args = d_args->next;
@@ -615,18 +640,18 @@ parse_rcfile(const char *line, int lineno)
      * namazurc.
      */
     if (nmz_is_debugmode() &&
-	(strcasecmp(directive, "COMMENT") != 0) &&
-	(strcasecmp(directive, "BLANK") != 0))
+        (strcasecmp(directive, "COMMENT") != 0) &&
+        (strcasecmp(directive, "BLANK") != 0))
     {
-	StrList *ptr;
-	int i;
+        StrList *ptr;
+        int i;
 
-	nmz_debug_printf("%4d: Directive:  [%s]\n", lineno, directive);
+        nmz_debug_printf("%4d: Directive:  [%s]\n", lineno, directive);
 
-	for (ptr = args, i = 1 ; ptr != NULL; i++) {
-	    nmz_debug_printf("      Argument %d: [%s]\n", i, ptr->value);
-	    ptr = ptr->next;
-	}
+        for (ptr = args, i = 1 ; ptr != NULL; i++) {
+            nmz_debug_printf("      Argument %d: [%s]\n", i, ptr->value);
+            ptr = ptr->next;
+        }
     }
 
     free_strlist(d_args);
@@ -641,35 +666,35 @@ apply_rc(int lineno, const char *directive, StrList *args)
     struct conf_directive *dtab = directive_tab;
 
     for (ptr = args; ptr != NULL; ptr = ptr->next) {
-	argnum++;
+        argnum++;
     }
 
     for (; dtab->name != NULL;  dtab++) {
-	if (strcasecmp(dtab->name, directive) == 0) {
-	    /* 
-	     * Check whether the number of argument is right.
-	     */
-	    if (argnum == dtab->argnum ||
-		(dtab->plus && argnum > dtab->argnum)) 
-	    {
-		/* 
-		 * If number of argument is correct, apply
-		 * appropriate function and return with its
-		 * return value.
-		 */
-		return dtab->func(directive, args);
-	    } else if (argnum < dtab->argnum) {
-		errmsg = _("too few arguments");
-		return FAILURE;
-	    } else if (argnum > dtab->argnum) {
-		errmsg = _("too many arguments");
-		return FAILURE;
-	    } else {
-		assert(0);
-		/* NOTREACHED */
-		return FAILURE;
-	    }
-	}
+        if (strcasecmp(dtab->name, directive) == 0) {
+            /* 
+             * Check whether the number of argument is right.
+             */
+            if (argnum == dtab->argnum ||
+                (dtab->plus && argnum > dtab->argnum)) 
+            {
+                /* 
+                 * If number of argument is correct, apply
+                 * appropriate function and return with its
+                 * return value.
+                 */
+                return dtab->func(directive, args);
+            } else if (argnum < dtab->argnum) {
+                errmsg = _("too few arguments");
+                return FAILURE;
+            } else if (argnum > dtab->argnum) {
+                errmsg = _("too many arguments");
+                return FAILURE;
+            } else {
+                assert(0);
+                /* NOTREACHED */
+                return FAILURE;
+            }
+        }
     }
 
     errmsg = _("unknown directive");
@@ -697,41 +722,41 @@ load_rcfile(const char *fname)
 
     fp = fopen(fname, "rb");
     if (fp == NULL) { /* open failed */
-	nmz_set_dyingmsg(nmz_msg("%s: %s", fname, strerror(errno)));
-	return FAILURE;
+        nmz_set_dyingmsg(nmz_msg("%s: %s", fname, strerror(errno)));
+        return FAILURE;
     }
 
     while (fgets(buf, BUFSIZE, fp) != NULL) {
-	int current_lineno = lineno;
-	
-	do {
-	    lineno++;
-	    nmz_chomp(buf);
-	    if (buf[strlen(buf) - 1] == '\\') { /* ending with \ */
-		int remaining;
-		
-		buf[strlen(buf) - 1] = '\0'; /* Remove ending \ */
-		remaining = BUFSIZE - strlen(buf) - 1;
-		if (remaining == 0) {
-		    nmz_chomp(buf);
-		    break;
-		}
-		if (fgets(buf + strlen(buf), remaining, fp) == NULL) {
-		    nmz_chomp(buf);
-		    break;
-		}
-	    } else {
-		break;
-	    }
-	} while (1);
+        int current_lineno = lineno;
 
-	nmz_codeconv_internal(buf);  /* for Shift_JIS encoding */
-	if (parse_rcfile(buf, current_lineno) != SUCCESS) {
-	    nmz_set_dyingmsg(nmz_msg(_("%s:%d: syntax error: %s"),  
+        do {
+            lineno++;
+            nmz_chomp(buf);
+            if (buf[strlen(buf) - 1] == '\\') { /* ending with \ */
+                int remaining;
+
+                buf[strlen(buf) - 1] = '\0'; /* Remove ending \ */
+                remaining = (int)(BUFSIZE - strlen(buf) - 1);
+                if (remaining == 0) {
+                    nmz_chomp(buf);
+                    break;
+                }
+                if (fgets(buf + strlen(buf), remaining, fp) == NULL) {
+                    nmz_chomp(buf);
+                    break;
+                }
+            } else {
+                break;
+            }
+        } while (1);
+
+        nmz_codeconv_internal(buf);  /* for Shift_JIS encoding */
+        if (parse_rcfile(buf, current_lineno) != SUCCESS) {
+            nmz_set_dyingmsg(nmz_msg(_("%s:%d: syntax error: %s"),  
 				     fname, current_lineno, errmsg));
             fclose(fp);
-	    return FAILURE;
-	}
+            return FAILURE;
+        }
     }
     fclose(fp);
 
@@ -798,55 +823,55 @@ load_rcfiles(void)
      *     - This can be overriden by environmentl variable NAMAZURC.
      */
     if (strcmp("all", namazunorc) != 0 && strcmp("sysconfig", namazunorc) != 0) {
-	char *env = getenv_namazurc();
-	if (env != NULL) {
-	    if (load_rcfile(env) != SUCCESS) {
-		return FAILURE;
-	    }
-	} else {
-	    char fname[BUFSIZE] = "";
-	    strncpy(fname, namazurcdir, BUFSIZE - 1);
-	    strncat(fname, "/namazurc",
-		    BUFSIZE - strlen(fname) - 1);
-	    /* 
-	 * Load the file only if it exists.
-	 */
-	    if (nmz_is_file_exists(fname)) {
-		if (load_rcfile(fname) != SUCCESS) {
-		    return FAILURE;
-		}
-	    }
-	}
+        char *env = getenv_namazurc();
+        if (env != NULL) {
+            if (load_rcfile(env) != SUCCESS) {
+                return FAILURE;
+            }
+        } else {
+            char fname[BUFSIZE] = "";
+            strncpy(fname, namazurcdir, BUFSIZE - 1);
+            strncat(fname, "/namazurc",
+                BUFSIZE - strlen(fname) - 1);
+            /* 
+             * Load the file only if it exists.
+             */
+            if (nmz_is_file_exists(fname)) {
+                if (load_rcfile(fname) != SUCCESS) {
+                    return FAILURE;
+                }
+            }
+        }
     }
 
     /*
      *  2. ~/.namazurc
      */
     if (strcmp("all", namazunorc) != 0 && strcmp("home", namazunorc) != 0) {
-	char *home = getenv("HOME");
-	if (home != NULL) {
-	    char fname[BUFSIZE] = "";
-	    strncpy(fname, home, BUFSIZE - 1);
-	    strncat(fname, "/.namazurc",
-		    BUFSIZE - strlen(fname) - 1);
-	    /* 
-	     * Load the file only if it exists.
-	     */
-	    if (nmz_is_file_exists(fname)) {
-		if (load_rcfile(fname) != SUCCESS) {
-		    return FAILURE;
-		}
-	    }
-	}
+        char *home = getenv("HOME");
+        if (home != NULL) {
+            char fname[BUFSIZE] = "";
+            strncpy(fname, home, BUFSIZE - 1);
+            strncat(fname, "/.namazurc",
+                BUFSIZE - strlen(fname) - 1);
+            /* 
+             * Load the file only if it exists.
+             */
+            if (nmz_is_file_exists(fname)) {
+                if (load_rcfile(fname) != SUCCESS) {
+                    return FAILURE;
+                }
+            }
+        }
     }
 
     /*
      *  3. user-specified namazurc set by namazu --config=file option.
      */
     if (user_namazurc[0] != '\0') {
-	if (load_rcfile(user_namazurc) != SUCCESS) {
-	    return FAILURE;
-	}
+        if (load_rcfile(user_namazurc) != SUCCESS) {
+            return FAILURE;
+        }
     }
 
     return SUCCESS;
@@ -858,10 +883,10 @@ show_config(void)
     int i;
 
     if (loaded_rcfiles.num >= 1) {
-	for (i =0; i < loaded_rcfiles.num; i++) {
-	    printf(_("Loaded rcfile: %s\n"), loaded_rcfiles.fnames[i]);
-	}
-	printf("--\n");
+        for (i =0; i < loaded_rcfiles.num; i++) {
+            printf(_("Loaded rcfile: %s\n"), loaded_rcfiles.fnames[i]);
+        }
+        printf("--\n");
     }
 
     printf(_("\
@@ -882,22 +907,22 @@ EmphasisTags: %s\t%s\n\
 
     /* Show aliases. */
     {
-	struct nmz_alias *list = nmz_get_aliases();
-	while (list) {
-	    printf(_("Alias:   %-20s\t%s\n"), 
-		   list->alias, list->real);
-	    list = list->next;
-	}
+        struct nmz_alias *list = nmz_get_aliases();
+        while (list) {
+            printf(_("Alias:   %-20s\t%s\n"), 
+                list->alias, list->real);
+            list = list->next;
+        }
     }
 
     /* Show replaces. */
     {
-	struct nmz_replace *list = nmz_get_replaces();
-	while (list) {
-	    printf(_("Replace: %-20s\t%s\n"), 
-		   list->pat, list->rep);
-	    list = list->next;
-	}
+        struct nmz_replace *list = nmz_get_replaces();
+        while (list) {
+            printf(_("Replace: %-20s\t%s\n"), 
+                list->pat, list->rep);
+            list = list->next;
+        }
     }
 
     /*    exit(0);*/
